"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.promiseCall = promiseCall;
exports.promiseReject = promiseReject;
exports.promiseResolve = promiseResolve;
exports.promiseDone = promiseDone;
exports.Async = void 0;

var _lodash = _interopRequireDefault(require("lodash"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 *
 * @type {{Promise: (function(*=): *)}}
 */
var Async = {
  Promise: function (_Promise) {
    function Promise(_x) {
      return _Promise.apply(this, arguments);
    }

    Promise.toString = function () {
      return _Promise.toString();
    };

    return Promise;
  }(function (call) {
    return new Promise(call);
  })
};
/**
 * Call a function and resolves as promise.
 *
 * @param call {function}
 * @returns {Promise}
 */

exports.Async = Async;

function promiseCall(call) {
  return Async.Promise(function (resolve, reject) {
    try {
      resolve(call());
    } catch (err) {
      reject(err);
    }
  });
}
/**
 * Reject with `err`.
 *
 * @param err
 * @returns {Promise}
 */


function promiseReject(err) {
  return Async.Promise(function (resolve, reject) {
    reject(err);
  });
}
/**
 * Resolve with `value`.
 *
 * @param value
 * @returns {Promise}
 */


function promiseResolve(value) {
  return Async.Promise(function (resolve) {
    resolve(value);
  });
}
/**
 * Call .done on promise if it exists.
 *
 * @param promise {Promise}
 */


function promiseDone(promise) {
  var done = promise && _lodash.default.isFunction(promise.done) ? promise.done : undefined;

  if (done) {
    promise.finally(function () {
      return done();
    });
  }
}

Async.reject = promiseReject;
Async.resolve = promiseResolve;
Async.fcall = promiseCall;
Async.done = promiseDone;